#include "NavigationStackFactory.h"

#include "ArmarXCore/core/exceptions/local/ExpressionException.h"

#include "Navigation/libraries/core/constants.h"
#include "Navigation/libraries/factories/GlobalPlannerFactory.h"
#include "Navigation/libraries/factories/LocalPlannerFactory.h"
#include "Navigation/libraries/factories/SafetyControllerFactory.h"
#include "Navigation/libraries/factories/TrajectoryControllerFactory.h"
#include "Navigation/libraries/server/NavigationStack.h"

namespace armarx::nav::fac
{
    server::NavigationStack
    NavigationStackFactory::create(const aron::datanavigator::DictNavigatorPtr& params,
                                   const core::Scene& ctx)
    {
        using aron::datanavigator::DictNavigator;

        const auto getElementOrNull =
            [&params](const core::StackLayer& layer) -> DictNavigator::PointerType
        {
            const std::string key = core::StackLayerNames.to_name(layer);

            if (params->hasElement(key))
            {
                return DictNavigator::DynamicCast(params->getElement(key));
            }

            ARMARX_INFO << "Skipping '" << key << "'";

            return nullptr;
        };

        const auto globalPlannerCfg = getElementOrNull(core::StackLayer::GlobalPlanner);
        const auto localPlannerCfg = getElementOrNull(core::StackLayer::LocalPlanner);
        const auto trajectoryControllerCfg =
            getElementOrNull(core::StackLayer::TrajectoryController);
        const auto safetyControllerCfg = getElementOrNull(core::StackLayer::SafetyController);

        return server::NavigationStack{
            .globalPlanner = GlobalPlannerFactory::create(globalPlannerCfg, ctx),
            .localPlanner = LocalPlannerFactory::create(localPlannerCfg, ctx),
            .trajectoryControl = TrajectoryControllerFactory::create(trajectoryControllerCfg, ctx),
            .safetyControl = SafetyControllerFactory::create(safetyControllerCfg, ctx)};
    }

    server::NavigationStack
    NavigationStackFactory::create(const aron::data::AronDictPtr& params, const core::Scene& ctx)
    {
        const auto dict = aron::datanavigator::DictNavigator::FromAronDictPtr(params);
        ARMARX_CHECK_NOT_NULL(dict);
        return create(dict, ctx);
    }

} // namespace armarx::nav::fac
